/* borrowedfrommondo.c

       subroutines which have been copied from
       mondo at some time or another :)
*/



#include <errno.h>
#include "tools.h"
#include "configure.h"
#include "progressform.h"
#include "borrowedfrommondo.h"
#include "libmondo-EXT.h"

extern void sensibly_set_tmpdir_and_scratchdir(struct s_bkpinfo *bkpinfo);

int find_kernel_path (char*);
void terminate_daemon(int sig);
void set_signals(int on);
void termination_in_progress(int sig);


pid_t g_main_pid;

extern long g_current_progress;

extern bool g_skip_floppies;
extern int g_currentY;
char g_mondo_home[MAX_STR_LEN];
bool g_text_mode=true;
char err_log_lines[NOOF_ERR_LINES][MAX_STR_LEN];



int find_kernel_path (char*path_sz)
{
  int res=0;

  strcpy(path_sz, call_program_and_get_last_line_of_output("mindi --findkernel"));
  if (!path_sz[0]) { res++; }
  return(res);
}





/* from mondo-cli.c */

void terminate_daemon(int sig)
{
  char tmp[64];
//  char command[512];
  pid_t pid;

  switch(sig)
    {
	case SIGINT:
		sprintf(tmp, "SIGINT");
		break;
	case SIGKILL:
		sprintf(tmp, "SIGKILL");
		break;
	case SIGTERM:
		sprintf(tmp, "SIGTERM");
		break;
	case SIGHUP:
		sprintf(tmp, "SIGHUP");
		break;
	case SIGSEGV:
		sprintf(tmp, "SIGSEGV");
		break;
	case SIGPIPE:
		sprintf(tmp, "SIGPIPE");
		break;
	default:
		sprintf(tmp, "(Unknown)");
    }
  strcat(tmp, " signal received from OS");
  log_to_screen(tmp);
  log_to_screen("Mondoarchive is terminating in response to a signal from the OS");
  if (getpid() == g_main_pid)
    {
      log_it("main PID is now calling finish()");
      pid = fork();
      if (pid>0)
        { sleep(2); run_program_and_log_output("for i in `ps ax | grep mondoarchive | grep \"[O|V]\" | cut -d' ' -f1`; do kill -9 $i ; done", TRUE); }
    }
  finish(254);
}





void set_signals(int on)
/*
Purpose:Turn on/off signal-trapping
Params:	on - turn on or off (true=on, false=off)
Return: None
*/
{
  int signals[]= { SIGKILL, SIGPIPE, SIGTERM, SIGHUP, SIGTRAP, SIGABRT, SIGINT, SIGSTOP, 0 };
  int i;
  for (i=0; signals[i]; i++)
    {
      if (on)
        { signal(signals[i], terminate_daemon); }
      else
        { signal(signals[i], termination_in_progress); }
    }
}





void termination_in_progress(int sig)
{
  log_it("Termination in progress");
  usleep(10000);
  pthread_exit(0);
}




void backup_SUB(void*params)
{
  int i;
  unsigned long lres;
  char tmp[MAX_STR_LEN];
  struct s_bkpinfo *bkpinfo;
  int res=0,retval=0;
  long diffs=0;
  char say_at_end[MAX_STR_LEN];

/* Initialize variables */
  say_at_end[0] = '\0';
  if (!(bkpinfo = (struct s_bkpinfo*)malloc(sizeof(struct s_bkpinfo)))) { fatal_error("Cannot malloc bkpinfo"); }

/* Configure the bkpinfo structure, global file paths, etc. */
  unmount_supermounts_if_necessary();
  log_it ("XMondo --- http://www.mondorescue.com --- backing up...");
  if (pre_param_configuration(bkpinfo)) { fatal_error("Failed to pre-param initialize"); }
  set_signals(TRUE);
  g_main_pid = getpid();
  register_pid(g_main_pid, "XMondo");

  copy_info_from_Configuration_struct_to_bkpinfo_struct(bkpinfo);
  bkpinfo->backup_data = true;
  sensibly_set_tmpdir_and_scratchdir(bkpinfo);
  if ((res+=post_param_configuration (bkpinfo))) { fatal_error("Bad parameters."); }
  bkpinfo->nonbootable_backup = true;

/* If we're meant to backup then backup */
  if (bkpinfo->backup_data)
    {
      res = backup_data(bkpinfo);
      retval += res;
      if (res)
	{ strcat (say_at_end, "Data archived. Please check the logs, just as a precaution. "); }
      else
	{ strcat (say_at_end, "Data archived OK. "); }
    }

/* If we're meant to verify then verify */
  if (bkpinfo->verify_data)
    {
      res = verify_data(bkpinfo);
      if (res<0)
        {
          sprintf(tmp, "%d differences found.", -res);
          log_to_screen(tmp);
          diffs=(long)(-res);
          res=0;
        }
      retval += res;
    }

/* Offer to write floppy disk images to physical disks */
  if (bkpinfo->backup_data && !g_skip_floppies)
    {
      res = offer_to_write_boot_floppies_to_physical_disks(bkpinfo);
      retval += res;
    }
  if (!retval)
    {
      mvaddstr_and_log_it (g_currentY++, 0,
			   "Backup and/or verify ran to completion. Everything appears to be fine.");
    }
  else
    {
      mvaddstr_and_log_it (g_currentY++, 0,
			 "Backup and/or verify ran to completion. However, errors did occur.");
    }
/*  sleep(3); */
  if (strlen(bkpinfo->scratchdir)>4)
    {
      sprintf (tmp, "rm -Rf %s", bkpinfo->scratchdir);
      system (tmp);
    }
  if (strlen(bkpinfo->tmpdir)>4)
    {
      sprintf (tmp, "rm -Rf %s", bkpinfo->tmpdir);
      system (tmp);
    }
  if (does_file_exist ("/root/images/mindi/mindi.iso"))
    {
      log_to_screen
	("/root/images/mindi/mindi.iso, a boot/utility CD, is available if you want it");
    }
  if (length_of_file ("/tmp/changed.files") > 2)
    {
          log_it("Type 'less /tmp/changed.files' to see which files don't match the archives");
          log_it("Calling popup_changelist_from_file()");
//          popup_changelist_from_file("/tmp/changed.files");
          log_it("Returned from popup_changelist_from_file()");
    }
  else
    {
      unlink ("/tmp/changed.files");
    }
  log_to_screen (say_at_end);
  run_program_and_log_output("rm -f /var/cache/mondo-archive/last-backup.aborted", FALSE);
  set_signals(FALSE);
  remount_supermounts_if_necessary();
  register_pid(0,"XMondo");
  finish(retval);
}






