//---------------------------------------------------------------------------

#include <clx.h>
#include <process.h>

#pragma hdrstop

#include "main.h"
#include "configure.h"
#include "tools.h"
#include "progressform.h"
#include "about.h"
#include "borrowedfrommondo.h"
#include "starting.h"
#include "libmondo-EXT.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.xfm"
TfrmMain *frmMain;
extern TfrmConfigure *frmConfigure;
char g_homedir[MAX_STR_LEN];
char g_mountlist_sz[MAX_STR_LEN];
bool g_mondoarchiveisrunning;
char g_xmondo_logfile[MAX_STR_LEN];

int call_mondoarchive_in_its_own_terminal(void);
int call_mondorestore_in_its_own_terminal(void);
bool popup_and_get_string(char*, char*, char*);
void popup_and_OK(char*);
extern struct s_Configuration g_configuration;
extern char g_config_file[MAX_STR_LEN];
int backup_backup_backup(void);
int compare_compare_compare(void);
int restore_restore_restore(void);
void sub_to_call_mondoarchive(void*);
void sub_to_call_mondorestore(void*);
void count_until_mondowindow_closes(void*);
void initialize_xmondo_at_start(void);
int ask_me_OK_or_cancel(char*);


//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuExitClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::mnuConfigureClick(TObject *Sender)
{
  if (g_mondoarchiveisrunning)
    { popup_and_OK("You may not reconfigure XMondo while\nan archive operation is in progress."); }
  else
    { frmConfigure->Show(); }
}
//---------------------------------------------------------------------------






void popup_and_OK(char*msg)
{
  Application->NormalizeTopMosts();
  Application->MessageBox( msg,
          "Alert", TMessageButtons() << smbOK, smsInformation,
          smbOK, smbCancel);
  Application->RestoreTopMosts();
  if (frmProgress->Visible)
    { frmProgress->BringToFront(); }
  Application->ProcessMessages();
}




int ask_me_yes_or_no(char*msg)
{
  int res;
  TMessageButtons Buttons;

  Buttons = TMessageButtons() << smbNo << smbYes;
  Application->NormalizeTopMosts();
  if (Application->MessageBox( msg,
          "Alert", Buttons, smsInformation,
          smbYes, smbNo) == smbYes)
    { res = 1; }
  else
    { res = 0; }
  Application->RestoreTopMosts();
  if (frmProgress->Visible)
    { frmProgress->BringToFront(); }
  Application->ProcessMessages();

  return(res);
}



int ask_me_OK_or_cancel(char*msg)
{
  int res;
  TMessageButtons Buttons;

  Buttons = TMessageButtons() << smbCancel << smbOK;
  Application->NormalizeTopMosts();
  if (Application->MessageBox( msg,
          "Alert", Buttons, smsInformation,
          smbOK, smbCancel) == smbOK)
    { res = 1; }
  else
    { res = 0; }
  Application->RestoreTopMosts();
  if (frmProgress->Visible)
    { frmProgress->BringToFront(); }
  Application->ProcessMessages();

  return(res);
}


bool popup_and_get_string(char*title, char*msg, char*output)
{
  AnsiString res_ansi;
  res_ansi = InputBox(title, msg, output); // output in this case is default string
  WideStringToPChar(output, res_ansi, MAX_STR_LEN);
  if (strlen(output)==0) { return(FALSE); } else { return(TRUE); }
}



//---------------------------------------------------------------------------



void __fastcall TfrmMain::mnuBackupClick(TObject *Sender)
{
  int res;

  if (g_mondoarchiveisrunning) { popup_and_OK("Mondoarchive is already running. Sorry."); return; }
  res = backup_backup_backup();
  if (res==32100)
    { popup_and_OK("Backup aborted, as you requested."); }
  else if (res)
    { popup_and_OK("Backup encountered a fatal error."); }
  else
    { popup_and_OK("Backup ran successfully."); }
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::mnuManpageClick(TObject *Sender)
{
  char command[MAX_STR_LEN];

  strcpy(command, find_home_of_exe("xterm"));
  strcat(command, " +cm +hold -e man mondoarchive &");
  system(command);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuAboutClick(TObject *Sender)
{
  frmAbout->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuHTMLManualClick(TObject *Sender)
{
  char command[512];
  char location_of_howto[512];

  strcpy(location_of_howto, call_program_and_get_last_line_of_output("find {/usr/share/doc,/usr/doc} -maxdepth 1 | grep XMondo | tail -n1"));
  strcat(location_of_howto, "/index.html");
  if (!does_file_exist(location_of_howto))
    { popup_and_OK("Cannot find HOWTO. Sorry!"); }
  else if (open_web_browser_in_background(location_of_howto))
    { popup_and_OK("Cannot find web browser. Sorry!"); }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------



void __fastcall TfrmMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
  if (CanClose && frmConfigure->Visible)
    {
      if (!ask_me_yes_or_no("Configuration editor is still open.\nAre you sure you want to exit?"))
       { CanClose = false; }
    }
  if (CanClose && g_mondoarchiveisrunning)
    {
      if (!ask_me_yes_or_no("Archive operation is still running.\nAre you sure you want to exit?"))
       { CanClose = false; }
    }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuCompareClick(TObject *Sender)
{
  int res;

  if (g_mondoarchiveisrunning) { popup_and_OK("Mondoarchive is already running. Sorry."); return; }
  res = compare_compare_compare();
  if (res==32100)
    { popup_and_OK("Compare aborted, as you requested."); }
  else if (res)
    { popup_and_OK("Compare encountered a fatal error."); }
  else
    { popup_and_OK("Compare ran successfully."); }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::mnuRestoreClick(TObject *Sender)
{
  int res;

  if (g_mondoarchiveisrunning) { popup_and_OK("Mondorestore is already running. Sorry."); return; }
  res = restore_restore_restore();
  if (res==32100)
    { popup_and_OK("Restore aborted, as you requested."); }
  else if (res)
    { popup_and_OK("Restore encountered a fatal error."); }
  else
    { popup_and_OK("Restore ran successfully."); }
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------

//---------------------------------------------------------------------------

void __fastcall TfrmMain::butBackupClick(TObject *Sender)
{
  int res;

  if (g_mondoarchiveisrunning) { popup_and_OK("Mondoarchive is already running. Sorry."); return; }
  res = backup_backup_backup();
  if (res==32100)
    { popup_and_OK("Backup aborted, as you requested."); }
  else if (res)
    { popup_and_OK("Backup encountered a fatal error."); }
  else
    { popup_and_OK("Backup ran successfully."); }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::butCompareClick(TObject *Sender)
{
  int res;

  if (g_mondoarchiveisrunning) { popup_and_OK("Mondoarchive is already running. Sorry."); return; }
  res = compare_compare_compare();
  if (res==32100)
    { popup_and_OK("Compare aborted, as you requested."); }
  else if (res)
    { popup_and_OK("Compare encountered a fatal error."); }
  else
    { popup_and_OK("Compare ran successfully."); }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::butRestoreClick(TObject *Sender)
{
  int res;

  if (g_mondoarchiveisrunning) { popup_and_OK("Mondorestore is already running. Sorry."); return; }
  res = restore_restore_restore();
  if (res==32100)
    { popup_and_OK("Restore aborted, as you requested."); }
  else if (res)
    { popup_and_OK("Restore encountered a fatal error."); }
  else
    { popup_and_OK("Restore ran successfully."); }
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------

void sub_to_call_mondoarchive(void*params)
{
  int *p_res;

  p_res = (int*)params;
  *p_res = call_mondoarchive_in_its_own_terminal();    // ...which will re-set res & cause counting to stop - cool!
  _endthread();
}




void sub_to_call_mondorestore(void*params)
{
  int *p_res;

  p_res = (int*)params;
  log_it("Calling call_mondorestore_in_its_own_terminal()");
  *p_res = call_mondorestore_in_its_own_terminal();    // ...which will re-set res & cause counting to stop - cool!
  _endthread();
}










void initialize_xmondo_at_start(void)
{
  char tmp[512];
  char command[MAX_STR_LEN];
  char mountlist_fname[MAX_STR_LEN];

  strcpy(g_homedir, call_program_and_get_last_line_of_output("echo $HOME"));
  sprintf(g_config_file, "%s/.XMondo.cfg", g_homedir);
  sprintf(g_xmondo_logfile, "%s/XMondo.log", g_homedir);
  log_it( "g_config_file = %s", g_config_file);
  log_it("---------------------------------------------------------");

  if (getuid() != 0)
    {
      popup_and_OK("Please run me as root.");
      exit(1);
    }
  if (!find_home_of_exe("mondoarchive")[0])
    {
      popup_and_OK("Please install Mondo.");
      exit(1);
    }
  if (!find_home_of_exe("mindi")[0])
    {
      popup_and_OK("Please install Mindi.");
      exit(1);
    }

  load_config_file(g_config_file);

  sprintf(mountlist_fname, "/tmp/mondo.mountlist.%d", (int)(random()%32767));
  sprintf(command, "mindi --makemountlist %s &> /dev/null", mountlist_fname);
  if (system(command)) { log_it( "Warning - failed to read mountlist from mindi"); }
  else
    {
      sprintf(command, "cat %s | grep -v -w nfs | tr -s '\t' ' ' | cut -d' ' -f1 | tr '\n' ' ' | awk '{print $0;}'", mountlist_fname);
      strcpy(g_mountlist_sz, call_program_and_get_last_line_of_output(command));
    }
  unlink(mountlist_fname);
}



void __fastcall TfrmMain::FormShow(TObject *Sender)
{
  frmStarting->Show();
  Application->ProcessMessages();
  sleep(1);
  initialize_xmondo_at_start();
  frmStarting->Hide();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::Button1Click(TObject *Sender)
{
  log_it("There.");


}
//---------------------------------------------------------------------------

